// Copyright (c) 2011 Oracle Inc.
//
// MySQL Connector/NET is licensed under the terms of the GPLv2
// <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most 
// MySQL Connectors. There are special exceptions to the terms and 
// conditions of the GPLv2 as it is applied to this software, see the 
// FLOSS License Exception
// <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published 
// by the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the Free Software Foundation, Inc., 
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA

using System;
using System.Collections;
using System.Data;
using System.Collections.Generic;
using MySql.Data.MySqlClient.Properties;
using System.Diagnostics;
using System.Text;
using System.Globalization;

namespace MySql.Data.MySqlClient
{
    internal class LoadBalancer
    {
        private static BaseLoadBalancer balancer;

        static LoadBalancer()
        {
            balancer = new BaseLoadBalancer();
        }

        public static string[] GetHosts(string connectionString)
        {
            lock (balancer)
                return balancer.GetHosts(connectionString);
        }

        public static bool IsBalancing(string connectionString)
        {
            lock (balancer)
                return balancer.IsBalancing(connectionString);
        }

        public static void ResetBalancing()
        {
            lock (balancer)
                balancer.Reset();
        }
    }

    struct HostStatus
    {
        public List<string> Hostnames;
        public int LastHostNameUsed;

        public HostStatus(string connectionString)
        {
            Hostnames = new List<string>();
            MySqlConnectionStringBuilder builder = new MySqlConnectionStringBuilder(connectionString);
            string[] hostnames = builder.Server.Split(',');
            foreach (string hostname in hostnames)
                Hostnames.Add(hostname);
            LastHostNameUsed = -1;
        }

        public string[] GetHosts()
        {
            string[] hosts = new string[Hostnames.Count];

            int index = LastHostNameUsed++;
            for (int i = 0; i < Hostnames.Count; i++)
            {
                index++;
                if (index == Hostnames.Count)
                    index = 0;
                hosts[i] = Hostnames[index];
            }
            return hosts;
        }

        public int NumHosts
        {
            get { return Hostnames.Count; }
        }
    }

    class BaseLoadBalancer
    {
        private Dictionary<string, HostStatus> hostStatus = new Dictionary<string,HostStatus>();

        public string[] GetHosts(string connectionString)
        {
            string key = GetKeyFromConnectionString(connectionString);
            if (!hostStatus.ContainsKey(key))
            {
                HostStatus s = new HostStatus(connectionString);
                if (s.NumHosts == 1) return s.GetHosts();
                hostStatus.Add(key, s);
            }
            HostStatus status = hostStatus[key];
            return status.GetHosts();
        }

        public void Reset()
        {
            hostStatus.Clear();
        }

        public bool IsBalancing(string connectionString)
        {
            string key = GetKeyFromConnectionString(connectionString);
            return hostStatus.ContainsKey(key);
        }

        private string GetKeyFromConnectionString(string connectionString)
        {
            MySqlConnectionStringBuilder settings = new MySqlConnectionStringBuilder(connectionString);
            string port = String.Empty;
            if (settings.ConnectionProtocol == MySqlConnectionProtocol.Tcp)
                port = settings.Port.ToString();
            else if (settings.ConnectionProtocol == MySqlConnectionProtocol.Memory)
                port = settings.SharedMemoryName;
            else if (settings.ConnectionProtocol == MySqlConnectionProtocol.NamedPipe)
                port = settings.PipeName;
            return String.Format("{0}:{1}", settings.Server, port);
        }

        private void AddHostStatus(string connectionString)
        {
            string key = GetKeyFromConnectionString(connectionString);
            
        }
    }

}